'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var emotionUtils = require('emotion-utils');
var isPropValid = _interopDefault(require('@emotion/is-prop-valid'));
var React = require('react');
var core = require('@emotion/core');
var utils = require('@emotion/utils');
var serialize = require('@emotion/serialize');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var testOmitPropsOnStringTag = emotionUtils.memoize(isPropValid);
var testOmitPropsOnComponent = function testOmitPropsOnComponent(key) {
  return key !== 'theme' && key !== 'innerRef';
};
var testAlwaysTrue = function testAlwaysTrue() {
  return true;
};
var omitAssign = function omitAssign(testFn, target) {
  var i = 2;
  var length = arguments.length;

  for (; i < length; i++) {
    var source = arguments[i];

    var _key = void 0;

    for (_key in source) {
      if (testFn(_key)) {
        target[_key] = source[_key];
      }
    }
  }

  return target;
};

function _render() {
  return core.consumer(this, this.renderChild);
}

var createStyled = function createStyled(tag, options) {
  if (process.env.NODE_ENV !== 'production') {
    if (tag === undefined) {
      throw new Error('You are trying to create a styled element with an undefined component.\nYou may have forgotten to import it.');
    }
  }

  var identifierName;

  if (options !== undefined) {
    identifierName = options.label;
  }

  var isReal = tag.__emotion_real === tag;
  var baseTag = isReal && tag.__emotion_base || tag;
  var omitFn = typeof baseTag === 'string' && baseTag.charAt(0) === baseTag.charAt(0).toLowerCase() ? testOmitPropsOnStringTag : testOmitPropsOnComponent;
  return function () {
    var args = arguments;
    var styles = isReal && tag[emotionUtils.STYLES_KEY] !== undefined ? tag[emotionUtils.STYLES_KEY].slice(0) : [];

    if (identifierName !== undefined) {
      styles.push("label:" + identifierName + ";");
    }

    if (args[0] == null || args[0].raw === undefined) {
      styles.push.apply(styles, args);
    } else {
      styles.push(args[0][0]);
      var len = args.length;
      var i = 1;

      for (; i < len; i++) {
        styles.push(args[i], args[0][i]);
      }
    }

    var Styled =
    /*#__PURE__*/
    function (_React$Component) {
      _inheritsLoose(Styled, _React$Component);

      function Styled(props) {
        var _this;

        _this = _React$Component.call(this, props) || this;

        _this.renderChild = function (context) {
          var className = '';
          var classInterpolations = [];
          _this.mergedProps = omitAssign(testAlwaysTrue, {}, _this.props, {
            theme: _this.props.theme || context.theme
          });

          if (typeof _this.props.className === 'string') {
            className += utils.getRegisteredStyles(context.registered, classInterpolations, _this.props.className);
          }

          var serialized = serialize.serializeStyles.call(_assertThisInitialized(_this), styles.concat(classInterpolations));
          var rules = utils.insertStyles(context, serialized);
          className += serialized.cls;
          var ele = React.createElement(baseTag, omitAssign(omitFn, {}, _this.props, {
            className: className,
            ref: _this.props.innerRef
          }));

          if (utils.shouldSerializeToReactTree && rules !== undefined) {
            return React.createElement(React.Fragment, null, React.createElement("style", {
              "data-emotion-ssr": serialized.name,
              dangerouslySetInnerHTML: {
                __html: rules
              }
            }), ele);
          }

          return ele;
        };

        return _this;
      }

      var _proto = Styled.prototype;
      _proto.render = _render;
      return Styled;
    }(React.Component); // $FlowFixMe


    Styled.displayName = identifierName !== undefined ? identifierName : "Styled(" + (typeof baseTag === 'string' ? baseTag : baseTag.displayName || baseTag.name || 'Component') + ")";
    var FinalStyled = React.forwardRef(function (props, ref) {
      if (ref === null) {
        // this avoids creating a new object if there's no ref
        return React.createElement(Styled, props);
      }

      return React.createElement(Styled, _extends({}, props, {
        innerRef: ref
      }));
    });
    FinalStyled.__emotion_real = FinalStyled;
    FinalStyled.__emotion_base = baseTag;
    FinalStyled.__emotion_styles = styles;

    FinalStyled.withComponent = function (nextTag, nextOptions) {
      return createStyled(nextTag, nextOptions !== undefined ? _extends({}, options, nextOptions) : options).apply(void 0, styles);
    };

    return FinalStyled;
  };
};

module.exports = createStyled;
//# sourceMappingURL=index.cjs.js.map
